%% This program is used to solve the moment of method and the characteristic mode problem, 
%    also to optimize the feeding position and get the feeding information, 
%      including position, amplitude and phase
%   steps: 1.Obtain the mesh generation information of the desired shape
%   using Matlab tools or something else;
%          2.Solve the MOM probelm;
%          3.Solve the CMs probelm;
%          4.Optimize the feeding position and get the feeding information;
%          5.Draw the current distribution.
clc;clear all;close all;
addpath(genpath(pwd));

%% data preparation
%Considered frequency Number of excitation positions
f=38e6;
% The electromagnetic parameters
epsilon_    =8.854e-012;
mu_         =1.257e-006;
% speed of light
c_          =1/sqrt(epsilon_*mu_);
eta_=sqrt(mu_/epsilon_);

load('mesh/Square');    %Load mesh generation information of the desired shape
[s1 s2]=size(p);    
if(s1==2)               
    p(3,:)=0;           
end

%% solve the MoM problem
TrianglesTotal=length(t);              
for m=1:TrianglesTotal 
   N=t(1:3,m);                        
   Vec1=p(:,N(1))-p(:,N(2));          
   Vec2=p(:,N(3))-p(:,N(2));           
   Area(m) =norm(cross(Vec1,Vec2))/2;  
   Center(:,m)=1/3*sum(p(:,N),2);    
end

Edge_=[];                     
n=0;                        
for m=1:TrianglesTotal
    N=t(1:3,m);             
    for k=m+1:TrianglesTotal  
        M=t(1:3,k);           
        a=1-all([N-M(1) N-M(2) N-M(3)]); 
        if(sum(a)==2)     
            n=n+1;        
            Edge_=[Edge_ M(find(a))]; 
            TrianglePlus(n)=m;     
            TriangleMinus(n)=k;       
        end
    end
end
EdgesTotal=length(Edge_);          

Edge__=[Edge_(2,:); Edge_(1,:)];
Remove=[];
for m=1:EdgesTotal
    Edge_m=repmat(Edge_(:,m),[1 EdgesTotal]);
    Ind1=any(Edge_  -Edge_m);   
    Ind2=any(Edge__ -Edge_m);
    A=find(Ind1.*Ind2==0);
    if(length(A)==3)           
        Out=find(t(4,TrianglePlus(A))==t(4,TriangleMinus(A)));
        Remove=[Remove A(Out)];
    end
end
Edge_(:,Remove)         =[];
TrianglePlus(Remove)    =[];
TriangleMinus(Remove)   =[];
EdgesTotal=length(Edge_);


for m=1:EdgesTotal   
   EdgeLength(m)=norm(p(:,Edge_(1,m))-p(:,Edge_(2,m))); 
end

IMT=[];
for m=1:TrianglesTotal
    n1=t(1,m);
    n2=t(2,m);
    n3=t(3,m);
    M=Center(:,m); 
    r1=    p(:,n1);
    r2=    p(:,n2);
    r3=    p(:,n3);
    r12=r2-r1;   
    r23=r3-r2;   
    r13=r3-r1;   
    C1=r1+(1/3)*r12; 
    C2=r1+(2/3)*r12; 
    C3=r2+(1/3)*r23; 
    C4=r2+(2/3)*r23;   
    C5=r1+(1/3)*r13;  
    C6=r1+(2/3)*r13; 
    a1=1/3*(C1+C5+r1); 
    a2=1/3*(C1+C2+M);
    a3=1/3*(C2+C3+r2);
    a4=1/3*(C2+C3+M);
    a5=1/3*(C3+C4+M);
    a6=1/3*(C1+C5+M);
    a7=1/3*(C5+C6+M);
    a8=1/3*(C4+C6+M);
    a9=1/3*(C4+C6+r3);
    Center_(:,:,m)=...
        [a1 a2 a3 a4 a5 a6 a7 a8 a9];
end

for m=1:EdgesTotal
    NoPlus=TrianglePlus(m); 
    n1=t(1,NoPlus);         
    n2=t(2,NoPlus);
    n3=t(3,NoPlus); 
    if((n1~=Edge_(1,m))&(n1~=Edge_(2,m))) NODE=n1; end;
    if((n2~=Edge_(1,m))&(n2~=Edge_(2,m))) NODE=n2; end;
    if((n3~=Edge_(1,m))&(n3~=Edge_(2,m))) NODE=n3; end;
    FreeVertex=p(:,NODE);
    
    RHO_Plus(:,m)   =+Center(:,NoPlus)-FreeVertex;  
    RHO__Plus(:,:,m)  =...
        +Center_(:,:,NoPlus)-repmat(FreeVertex,[1 9]);
end

for m=1:EdgesTotal
    NoMinus=TriangleMinus(m);
    n1=t(1,NoMinus);
    n2=t(2,NoMinus);
    n3=t(3,NoMinus);
    if((n1~=Edge_(1,m))&(n1~=Edge_(2,m))) NODE=n1; end;
    if((n2~=Edge_(1,m))&(n2~=Edge_(2,m))) NODE=n2; end;
    if((n3~=Edge_(1,m))&(n3~=Edge_(2,m))) NODE=n3; end;
    FreeVertex=p(:,NODE);
    
    RHO_Minus(:,m)   =-Center(:,NoMinus) +FreeVertex;
    RHO__Minus(:,:,m)=...
        -Center_(:,:,NoMinus)+repmat(FreeVertex,[1 9]);
end

V=zeros(EdgesTotal,1);
omega       =2*pi*f;
k0          =omega/c_;
K           =1i*k0;
omega       =2*pi*f;
Constant1   =mu_/(4*pi);
Constant2   =-1/(4*pi*omega*epsilon_);
Factor      =1/9;
FactorA     =Factor*(omega*EdgeLength/4)*Constant1;  
FactorFi    =Factor*EdgeLength*Constant2;

for m=1:EdgesTotal
    RHO_P(:,:,m)=repmat(RHO_Plus(:,m),[1 9]);   %[3 9 EdgesTotal]  
    RHO_M(:,:,m)=repmat(RHO_Minus(:,m),[1 9]);  %[3 9 EdgesTotal]
end
FactorA=FactorA.';  
FactorFi=FactorFi.';
    
[Xe]=  impmetXe( EdgesTotal,TrianglesTotal,...
    EdgeLength,K,...
    Center,Center_,...
    TrianglePlus,TriangleMinus,...
    RHO_P,RHO_M,...
    RHO__Plus,RHO__Minus,...
    FactorA,FactorFi);
[Xm]=  impmetXm( EdgesTotal,TrianglesTotal,...
    EdgeLength,K,...
    Center,Center_,...
    TrianglePlus,TriangleMinus,...
    RHO_P,RHO_M,...
    RHO__Plus,RHO__Minus,...
    FactorA,FactorFi);

Z=  impmet( EdgesTotal,TrianglesTotal,...
    EdgeLength,K,...
    Center,Center_,...
    TrianglePlus,TriangleMinus,...
    RHO_P,RHO_M,...
    RHO__Plus,RHO__Minus,...
    FactorA*1i,-FactorFi/1i);

%% solve the CMs problems
Z=(Z+Z.')/2;
R=real(Z);
Xe=(Xe+Xe.')/4;
Xm=(Xm+Xm.')/4;
    
[v,d]=eig(R,Xe+Xm);
d=sum(d,1);
Q=abs(1./d);
[Qsort,seq]=sort(Q);

%% feeding algorithm
ExNum=seq(1);    % the first mode is the desired mode
Isti=v(:,ExNum);  
Asti=Isti'*R*((Z)\diag(EdgeLength));
Asti=abs(Asti);
[ans,seqAsti]=sort(Asti,'descend');
NumofSti=floor(EdgesTotal*0.05);   %Number of excitation positions
Qsti=[];  
Relation=[]; 
Jodemain=v(:,seq(1:9));   
BestQ=[];  %Best Q
BestI=[]; 
BestV=zeros(EdgesTotal,2);
for i=1:NumofSti
    V=zeros(EdgesTotal,1);
    V(seqAsti(i))=1*EdgeLength(seqAsti(i));   
    I(:,i) = Z\V;
    Qsti(i)=(I(:,i)'*(Xe+Xm)*I(:,i))/(I(:,i)'*R*I(:,i)); 
    Coefficient=abs(Jodemain\I(:,i));
    Relation(i,:)=Coefficient/Coefficient(find(seq==ExNum));
end

for i=1:NumofSti
    Wp=find(real(Qsti)<0);
    Qsti(Wp)=inf;  
    Relation(Wp,:)=inf*Relation(Wp,:);
end

[ans,BestSeq]=min(abs(Qsti));
BestQ(1)=ans;
BestV(seqAsti(BestSeq),1)=1; 
BestI(:,1)=I(:,BestSeq); 

Qsti1=[]; 
Relation1=[]; 
V1=[];
Sign=size(find(Relation(BestSeq,:)>0.6+1e-6),2);
if(Sign==0)
    NULL;
else   %mode inhibited
    for i=1:NumofSti
        NumRest=find(Relation(i,:)>0.5+1e-6); 
        a=[1;1e-6*ones(size(NumRest,2)-1,1)]; 
        Jmod1=v(:,ExNum);  
        for j=1:1:size(NumRest,2)
            if(NumRest(j)==find(seq==ExNum))
               continue;
            else
                Jmod1(:,end+1)=Jodemain(:,NumRest(j));
            end
        end
        A1=Jmod1'*R*((Z)\diag(EdgeLength));
        V1(:,i)=A1\a;   
        V1(:,i)=(V1(:,i)/max(abs(V1(:,i))));
        I1(:,i) = Z\(V1(:,i).*EdgeLength');
        Qsti1(i)=(I1(:,i)'*(Xe+Xm)*I1(:,i))/(I1(:,i)'*R*I1(:,i));   
        Se=find(V1(:,i)~=0);
        for ii=1:size(Se,1)
            FeedPower(i,ii)=1/2*real(I1(Se(ii),i)*conj(V1(Se(ii),i)));
            Impedance(i,ii)=V1(Se(ii),i)/(I1(Se(ii),i)*EdgeLength(Se(ii)));
        end
        Coefficient=abs(Jodemain\I1(:,i));
        Relation1(i,:)=Coefficient/Coefficient(find(seq==ExNum));
    end
end

for i=1:NumofSti
    Wp=find(real(Qsti1)<0);
    Qsti1(Wp)=inf; 
    Relation1(Wp,:)=inf*Relation1(Wp,:);
end

[ans,BestSeq1]=min(abs(Qsti1));
BestQ(2)=ans;
BestV(:,2)=V1(:,BestSeq1);
BestI(:,2)=I1(:,BestSeq1);    

figure(1)
plot(1:EdgesTotal,BestV(:,1));
xlabel('Triangular basis number','fontsize',13)
ylabel('Feed voltage/V','fontsize',13)
figure(2)
plot(1:EdgesTotal,BestV(:,2));
xlabel('Triangular basis number','fontsize',13)
ylabel('Feed voltage/V','fontsize',13)

PositionV1=[];
SS=find(BestV(:,1)>0);
Node1=Edge_(1,SS);
Node2=Edge_(2,SS);
PositionV1(1,:)=p(:,Node1);
PositionV1(2,:)=p(:,Node2);
% Impedance1=BestV(SS,1)/(BestI(SS,1)*EdgeLength(SS))

Impedance2=[]; 
PositionV2=[];
ManyV=[];
SS1=find(BestV(:,2)~=0);
for i=1:1:size(SS1,1)
    Node1=Edge_(1,SS1(i));
    Node2=Edge_(2,SS1(i));
    PositionV2(i,1:3)=p(:,Node1);
    PositionV2(i,4:6)=p(:,Node2);
    ManyV(1,i)=abs(BestV(SS1(i),2));
    ManyV(2,i)=angle(BestV(SS1(i),2))*180/pi;
end
ManyV(1,:)=ManyV(1,:)/max(ManyV(1,:));
II=Z\(BestV(:,2)/max(abs(BestV(:,2))).*EdgeLength');
Qnew=II'*(Xe+Xm)*II/(II'*(R)*II)
Impedance2(1)=BestV(SS1(1),2)/II(SS1(1));
Impedance2(2)=BestV(SS1(2),2)/II(SS1(2));

if BestQ(1)>BestQ(2)
    S=2;
else
    S=1;
end
%% draw
Index=find(t(4,:)<=1);
Triangles=length(Index);
CurrentVector=[];
CurrentNorm=[];
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=BestI(m,S)*EdgeLength(m);
        if(TrianglePlus(m)==k)  
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
    CurrentNorm(k)=abs(norm(i));  
    CurrentVector(1:2,k)=i(1:2);
end

Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]') 
CurrentNorm1=CurrentNorm/max(CurrentNorm);
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
end
C=repmat(CurrentNorm1,3,1);
figure(5)
h=fill(XX,YY,C);
colormap gray;
colormap(flipud(colormap))
axis('equal');
rotate3d
hold on;
quiver(Center(1,:),Center(2,:),CurrentVector(1,:),CurrentVector(2,:),0.3,'color', 'r', 'linewidth', 1.5,'maxheadsize',0.3)
xlabel('y,m','fontsize',13)
ylabel('x,m','fontsize',13)

Index=find(t(4,:)<=1);
Triangles=length(Index);
CurrentVector=[];
CurrentNorm=[];
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=v(m,seq(1))*EdgeLength(m);
        if(TrianglePlus(m)==k)  
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
    CurrentNorm(k)=abs(norm(i));  
    CurrentVector(1:2,k)=i(1:2);
end

Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]')
CurrentNorm1=CurrentNorm/max(CurrentNorm);
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
end
C=repmat(CurrentNorm1,3,1);
figure(6)
h=fill(XX,YY,C); 
colormap gray;
colormap(flipud(colormap))
axis('equal');
rotate3d
hold on;
quiver(Center(1,:),Center(2,:),CurrentVector(1,:),CurrentVector(2,:),1.5,'color', 'r', 'linewidth', 1.5,'maxheadsize',1.5)
xlabel('y,m','fontsize',13)
ylabel('x,m','fontsize',13)


Index=find(t(4,:)<=1);
Triangles=length(Index);
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=v(m,seq(1))*EdgeLength(m);
        if(TrianglePlus(m)==k)  
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
     CurrentNorm(:,k)=norm(i);      
end
Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]') 
CurrentNorm1=CurrentNorm/max(CurrentNorm);
l=[];
r0=[0.53,-0.53];
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
    Cen(1,m)=mean(XX(:,m));
    Cen(2,m)=mean(YY(:,m));

    if(Cen(1,m)>0.525&&Cen(2,m)>-0.525)
        l(m)=Cen(2,m)+0.525;
    elseif(Cen(1,m)<0.525&&Cen(2,m)>0.525)
        l(m)=(-Cen(1,m)+0.525)+1.055;
    elseif(Cen(1,m)<-0.525&&Cen(2,m)<0.525)
        l(m)=(0.53-Cen(2,m))+2.11;
    elseif(Cen(1,m)>-0.525&&Cen(2,m)<-0.525)
        l(m)=(Cen(1,m)+0.53)+3.165;
    else
        l(m)=0;
    end
    
end
figure(7)
plot(l/(c_/f),CurrentNorm1,'o')
xlabel('s/{\lambda}','fontsize',13)
ylabel('Normalized I','fontsize',13)


Index=find(t(4,:)<=1);
Triangles=length(Index);
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=BestI(m,S)*EdgeLength(m);
        if(TrianglePlus(m)==k)  
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
     CurrentNorm(:,k)=norm(i);      
end
Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]') 
CurrentNorm1=CurrentNorm/max(CurrentNorm);
l=[];
r0=[0.53,-0.53];
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
    Cen(1,m)=mean(XX(:,m));
    Cen(2,m)=mean(YY(:,m));

    if(Cen(1,m)>0.525&&Cen(2,m)>-0.525)
        l(m)=Cen(2,m)+0.525;
    elseif(Cen(1,m)<0.525&&Cen(2,m)>0.525)
        l(m)=(-Cen(1,m)+0.525)+1.055;
    elseif(Cen(1,m)<-0.525&&Cen(2,m)<0.525)
        l(m)=(0.53-Cen(2,m))+2.11;
    elseif(Cen(1,m)>-0.525&&Cen(2,m)<-0.525)
        l(m)=(Cen(1,m)+0.53)+3.165;
    else
        l(m)=0;
    end
    
end
figure(8)
plot(l/(c_/f),CurrentNorm1,'o')
xlabel('s/{\lambda}','fontsize',13)
ylabel('Normalized I','fontsize',13)

Index=find(t(4,:)<=1);
Triangles=length(Index);
CurrentVector=[];
CurrentNorm=[];
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=BestI(m,1)*EdgeLength(m);
        if(TrianglePlus(m)==k) 
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
    CurrentNorm(k)=abs(norm(i));  
    CurrentVector(1:2,k)=i(1:2);
end

Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]')
CurrentNorm1=CurrentNorm/max(CurrentNorm);
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
end
C=repmat(CurrentNorm1,3,1);
figure(9)
h=fill(XX,YY,C);
colormap gray;
colormap(flipud(colormap))
axis('equal');
rotate3d
hold on;
quiver(Center(1,:),Center(2,:),CurrentVector(1,:),CurrentVector(2,:),0.5,'color', 'r', 'linewidth', 1.5,'maxheadsize',1.2)
xlabel('y,m','fontsize',13)
ylabel('x,m','fontsize',13)

Index=find(t(4,:)<=1);
Triangles=length(Index);
for k=1:Triangles
    i=[0 0 0]';
    for m=1:EdgesTotal
        IE=BestI(m,1)*EdgeLength(m);
        if(TrianglePlus(m)==k) 
            i=i+IE*RHO_Plus(:,m)/(2*Area(TrianglePlus(m)));
        end
        if(TriangleMinus(m)==k)
            i=i+IE*RHO_Minus(:,m)/(2*Area(TriangleMinus(m)));
        end
    end
     CurrentNorm(:,k)=norm(i);      
end
Jmax=max(CurrentNorm);
MaxCurrent=strcat(num2str(Jmax),'[A/m]')
CurrentNorm1=CurrentNorm/max(CurrentNorm);
l=[];
r0=[0.53,-0.53];
for m=1:Triangles
    N=t(1:3,m);
    XX(1:3,m)=[p(1,N)]';
    YY(1:3,m)=[p(2,N)]';  
    Cen(1,m)=mean(XX(:,m));
    Cen(2,m)=mean(YY(:,m));

    if(Cen(1,m)>0.525&&Cen(2,m)>-0.525)
        l(m)=Cen(2,m)+0.525;
    elseif(Cen(1,m)<0.525&&Cen(2,m)>0.525)
        l(m)=(-Cen(1,m)+0.525)+1.055;
    elseif(Cen(1,m)<-0.525&&Cen(2,m)<0.525)
        l(m)=(0.53-Cen(2,m))+2.11;
    elseif(Cen(1,m)>-0.525&&Cen(2,m)<-0.525)
        l(m)=(Cen(1,m)+0.53)+3.165;
    else
        l(m)=0;
    end
    
end
figure(10)
plot(l/(c_/f),CurrentNorm1,'o')
xlabel('s/{\lambda}','fontsize',13)
ylabel('Normalized I','fontsize',13)
